#include "stm8l.h"
#include "stm8l_gpio.h"
#include "stm8l_clk.h"
#include "stm8l_tim4.h"
#include "stm8l_usart.h"
#include "stdio.h"

#define PORT GPIOB
#define LED  GPIO_Pin_1
#define TIM4_PERIOD 124

#define PUTCHAR_PROTOTYPE void putchar (char c)

volatile uint16_t count;
uint8_t i;

INTERRUPT_HANDLER(IRQ_Handler_TIM4, 25)
{
	if (count)
		count--;

	TIM4_ClearITPendingBit(TIM4_IT_Update);
}

void delay_ms(uint16_t ms)
{
        TIM4_Cmd(DISABLE);       // stop

        TIM4_TimeBaseInit(TIM4_Prescaler_128, TIM4_PERIOD);
        TIM4_ClearFlag(TIM4_FLAG_Update);
        TIM4_ITConfig(TIM4_IT_Update, ENABLE);

        count = ms;

        TIM4_Cmd(ENABLE);       // let's go

 	while(count);
}

int main( void )
{
	// ---------   GPIO Config ---------------------------

	GPIO_DeInit(PORT);

	GPIO_Init(PORT, LED, GPIO_Mode_Out_PP_Low_Fast);

        // ---------- CLK Config -----------------------
        CLK_DeInit();
        CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);
	CLK_PeripheralClockConfig(CLK_Peripheral_TIM4, ENABLE);
	CLK_PeripheralClockConfig(CLK_Peripheral_USART1, ENABLE);
	// ---------- TIM4 Init -----------------------------
	TIM4_DeInit();

	// ---------- USART Init ---------------------------
	USART_DeInit(USART1);

  	USART_Init(USART1, (uint32_t)9600, USART_WordLength_8b, USART_StopBits_1,
                   USART_Parity_No, (USART_Mode_TypeDef)(USART_Mode_Tx | USART_Mode_Rx));

	enableInterrupts();
	i=0;

	for(;;)
  	{
    		delay_ms(1000);
		GPIO_ToggleBits(PORT, LED);
		printf("count: %d\n",++i);
  	}

}

PUTCHAR_PROTOTYPE
{
	/* Write a character to the USART */
	USART_SendData8(USART1, c);
	/* Loop until the end of transmission */
  	while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);

//  	return (c);
}
