#include "stm8s.h"
#include "stm8s_gpio.h"
#include "stm8s_clk.h"
#include "stm8s_adc1.h"
#include "stm8s_tim4.h"
#include "stm8s_uart1.h"
#include "stdio.h"

#define LED_PORT GPIOB
#define LED GPIO_PIN_5

#define TIM4_PERIOD       124

#define PUTCHAR_PROTOTYPE void putchar (char c)

ErrorStatus status = FALSE;
volatile uint16_t count;
volatile uint16_t adc2;
volatile uint16_t arr[20];

uint8_t i;

INTERRUPT_HANDLER(IRQ_Handler_TIM4, 23)
{
	if (count)
		count--;

  	TIM4_ClearITPendingBit(TIM4_IT_UPDATE);
}

INTERRUPT_HANDLER(IRQ_Handled_ADC1,22)
{
	adc2=ADC1_GetConversionValue();

	ADC1_ClearITPendingBit(ADC1_IT_EOC);
}

void delay_ms(uint16_t ms)
{
        TIM4_Cmd(DISABLE);       // stop
        TIM4_TimeBaseInit(TIM4_PRESCALER_128, TIM4_PERIOD);
        TIM4_ClearFlag(TIM4_FLAG_UPDATE);
        TIM4_ITConfig(TIM4_IT_UPDATE, ENABLE);

	count = ms;

        TIM4_Cmd(ENABLE);       // let's go

	while(count);
}


int main( void )
{

	// ----------- GPIO CONFIG -------------------
	GPIO_DeInit(LED_PORT);
	GPIO_Init(LED_PORT, LED, GPIO_MODE_OUT_PP_LOW_FAST);

	// ---------- CLK CONFIG -----------------------
	CLK_DeInit();

    	CLK_SYSCLKConfig(CLK_PRESCALER_CPUDIV1);
	CLK_SYSCLKConfig(CLK_PRESCALER_HSIDIV1); // set 16 MHz for CPU

	TIM4_DeInit();
	// uncomment if use HSE on Quartz
	//status = CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSE, DISABLE,
        //                CLK_CURRENTCLOCKSTATE_DISABLE);

	// ------------ ADC1 -------------------
	GPIO_Init(GPIOC,GPIO_PIN_4,GPIO_MODE_IN_FL_NO_IT);
	ADC1_DeInit();
	ADC1_Init(ADC1_CONVERSIONMODE_SINGLE, ADC1_CHANNEL_2, ADC1_PRESSEL_FCPU_D8,
			ADC1_EXTTRIG_TIM,DISABLE, ADC1_ALIGN_RIGHT,  ADC1_SCHMITTTRIG_CHANNEL2, DISABLE);
    	ADC1_ITConfig(ADC1_IT_EOCIE ,ENABLE);
//    	ADC1_StartConversion();

	// ------------ UART1 -------------------
	UART1_DeInit();

	// 8N1
	UART1_Init((uint32_t)9600, UART1_WORDLENGTH_8D, UART1_STOPBITS_1, UART1_PARITY_NO,
              UART1_SYNCMODE_CLOCK_DISABLE, UART1_MODE_TXRX_ENABLE);


	adc2=0;
	enableInterrupts();

	for(;;)
  	{
		GPIO_WriteReverse(LED_PORT, LED);
		for (i=0; i<20; i++)
		{
	    		ADC1_StartConversion();
			delay_ms(1);
			arr[i]=adc2;
		}
		delay_ms(1000);
		for (i=0;i<20;i++) 
		{
			printf(" %u ",arr[i]);
		}
		printf("\n");
		delay_ms(1000);
  	}
}


PUTCHAR_PROTOTYPE
{
  	// Write a character to the UART1
  	UART1_SendData8(c);
  	// Loop until the end of transmission 
  	while (UART1_GetFlagStatus(UART1_FLAG_TXE) == RESET);
 }
