#include "stm8s.h"
#include "stm8s_gpio.h"
#include "stm8s_clk.h"
#include "stm8s_tim4.h"
#include "stm8s_tim2.h"

#define LED_PORT GPIOB
#define LED GPIO_PIN_5

#define TIM4_PERIOD       124

volatile uint16_t count;

INTERRUPT_HANDLER(IRQ_Handler_TIM4, 23)
{
	if (count)
		count--;

  	TIM4_ClearITPendingBit(TIM4_IT_UPDATE);
}

void delay_ms(uint16_t ms)
{
        TIM4_Cmd(DISABLE);       // stop
        TIM4_TimeBaseInit(TIM4_PRESCALER_128, TIM4_PERIOD);
        TIM4_ClearFlag(TIM4_FLAG_UPDATE);
        TIM4_ITConfig(TIM4_IT_UPDATE, ENABLE);

	count = ms;

        TIM4_Cmd(ENABLE);       // let's go

	while(count);
}



ErrorStatus status = FALSE;


int main( void )
{

	// ----------- GPIO CONFIG -------------------
	GPIO_DeInit(LED_PORT);
	GPIO_Init(LED_PORT, LED, GPIO_MODE_OUT_PP_LOW_FAST);

	// ---------- CLK CONFIG -----------------------
	CLK_DeInit();

    	CLK_SYSCLKConfig(CLK_PRESCALER_CPUDIV1);
	CLK_SYSCLKConfig(CLK_PRESCALER_HSIDIV1); // set 16 MHz for CPU

	TIM4_DeInit();
	// uncomment if use HSE on Quartz
	//status = CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSE, DISABLE,
        //                CLK_CURRENTCLOCKSTATE_DISABLE);

	TIM2_DeInit();

	TIM2_TimeBaseInit(TIM2_PRESCALER_1, 999);

  	// PWM1 Mode configuration: Channel1
  	TIM2_OC1Init(TIM2_OCMODE_PWM1, TIM2_OUTPUTSTATE_ENABLE,500, TIM2_OCPOLARITY_HIGH);
  	TIM2_OC1PreloadConfig(ENABLE);

	TIM2_ARRPreloadConfig(ENABLE);

	TIM2_Cmd(ENABLE);

	enableInterrupts();


	for(;;)
  	{
		// 90% PWM
		delay_ms(1000);
		TIM2_SetCompare1(900);
		GPIO_WriteHigh(LED_PORT, LED);
		delay_ms(1000);
		GPIO_WriteLow(LED_PORT, LED);

		// 50% PWM
		delay_ms(1000);
		TIM2_SetCompare1(500);
		GPIO_WriteReverse(LED_PORT, LED);
                delay_ms(1000);
                GPIO_WriteLow(LED_PORT, LED);

		// 10% PWM
		delay_ms(1000);
		TIM2_SetCompare1(100);
		GPIO_WriteReverse(LED_PORT, LED);
                delay_ms(1000);
                GPIO_WriteLow(LED_PORT, LED);
  	}
}
